<?php

require_once 'ManageDashboardsWithTeams.php';

class ConfigureUserDashboardHook
{
    public function configureUserDashboard($bean, $event, $arguments)
    {
        if (($bean->is_team == true) || ($bean->is_team == 1) || ($bean->is_team == '1')) {
            $manageTeams = new ManageDashboardsWithTeams();
            $manageTeams->configureRecordsWithTeam($bean);
        } else {
            $this->configureRecordsWithUser($bean, $arguments);
        }
    }

    private function configureRecordsWithUser($bean, $arguments)
    {
        $enabledModules = array('Accounts', 'Contacts', 'Leads', 'Opportunities');
        $changedModules = array_keys($arguments['dataChanges']);
        $results = $this->getUserDashboards($bean->user_id_c);

        foreach ($enabledModules as $module) {

            // Only update record for the changed modules.
            if (in_array(strtolower($module), $changedModules)) {
                $this->handleDashboard($module, $bean, $results, $bean->user_id_c);
            }
        }
    }

    private function handleDashboard($moduleName, $bean, $existingDashboards, $userId)
    {
        $currentModule = array();
        if (empty($userId)) {
            if (empty($bean->user_id_c)) {
                return;
            }
            $userId = $bean->user_id_c;
        }

        if (!empty($existingDashboards)) {
            foreach ($existingDashboards as $result) {
                if ($result['dashboard_module'] == $moduleName) {
                    $currentModule['id'] = $result['id'];
                    $currentModule['name'] = $result['dashboard_module'];
                }
            }
        }

        $beanModuleName = strtolower($moduleName);
        if ($bean->$beanModuleName == "1") {
            $this->createDashboard($moduleName, $userId);
            return;
        }

        if ($bean->$beanModuleName == "0" && !empty($currentModule)) {
            $this->disableDashboard($currentModule['id'], $userId);
        }
    }

    private function createDashboard($module, $userId)
    {
        $user = BeanFactory::getBean("Users", $userId);

        $bean = BeanFactory::newBean('Dashboards');
        $bean->dashboard_module = $module;
        $bean->assigned_user_id = $userId;
        $bean->view_name = 'record';
        $bean->name = 'LBL_LINKEDIN_NAVIGATOR';
        $bean->modified_user_id = $userId;
        $bean->created_by = $userId;

        $bean->deleted = 0;
        $bean->metadata = '{"components":[{"rows":[[{"width":12,"view":{"label":"LinkedIn Sales Navigator","type":"linkedin-navigator","module":null}}]],"width":12}]}';
        $bean->team_id = $user->getPrivateTeam();

        $bean->setCreateData(false, $user);
        $bean->set_created_by = false;

        if($bean->id){
            $bean->new_with_id = true;
        }

        $bean->save();
        $this->updateDashboardAssignedUserId($bean, $userId);


        $this->toggleFavorite($bean, true, $user);
    }

    private function disableDashboard($dashboardId, $userId)
    {
        $bean = BeanFactory::getBean('Dashboards', $dashboardId);
        $bean->mark_deleted($bean->id);
    }

    /**
     * Retrieves all the configured dashboards of interest.
     * Uses prepared statements.
     * SugarQuery() doesn't work because of an "assigned_user_id=1" condition.
     *
     * @param $userId
     * @return array
     * @throws \Doctrine\DBAL\DBALException
     */
    private function getUserDashboards($userId)
    {
        $results = array();
        $query = "SELECT
              dashboards.id,
              dashboards.dashboard_module
            FROM dashboards
            WHERE dashboards.assigned_user_id = ? AND dashboards.name = ? and deleted=0";
        $db = DBManagerFactory::getInstance();
        $conn = $db->getConnection();
        $stmt = $conn->executeQuery($query, array($userId, 'LBL_LINKEDIN_NAVIGATOR'));
        foreach ($stmt->fetchAll() as $row) {
            $results[] = $row;
        }

        return $results;
    }

    /**
     * We need to update the assigned_user_id after save because it is overridden with $GLOBALS['current_user'].
     *
     * @param $bean
     * @param $newUserId
     */
    private function updateDashboardAssignedUserId($bean, $newUserId)
    {
        // For Sugar 7.8 (legacy)
        $query = "UPDATE dashboards SET assigned_user_id = '" . $newUserId . "' WHERE id = '" . $bean->id . "'";
        $GLOBALS['db']->query($query);
    }

    protected function toggleFavorite(SugarBean $bean, $favorite, $user)
    {

        $reindexBean = false;

        $favorite = (bool) $favorite;

        $module = "Dashboards";
        $record = $bean->id;

        $fav_id = SugarFavorites::generateGUID($module,$record);

        // get it even if its deleted
        $fav = BeanFactory::getBean('SugarFavorites', $fav_id, array("deleted" => false));

        // already exists
        if(!empty($fav->id)) {
            $deleted = ($favorite) ? 0 : 1;
            $fav->toggleExistingFavorite($fav_id, $deleted);
            $reindexBean = true;
        }

        elseif($favorite && empty($fav->id)) {
            $fav = BeanFactory::newBean('SugarFavorites');
            $fav->id = $fav_id;
            $fav->new_with_id = true;
            $fav->module = $module;
            $fav->record_id = $record;
            $fav->assigned_user_id = $user->id;
            $fav->deleted = 0;
            $fav->setCreateData(false, $user);
            $fav->set_created_by = false;

            $fav->save();
        }

        $bean->my_favorite = $favorite;

        // Bug59888 - If a Favorite is toggled, we need to reindex the bean for FTS engines so that the document will be updated with this change
        if($reindexBean === true) {
            $searchEngine = SugarSearchEngineFactory::getInstance(SugarSearchEngineFactory::getFTSEngineNameFromConfig());

            if($searchEngine instanceof SugarSearchEngineAbstractBase) {
                $searchEngine->indexBean($bean, false);
            }
        }

        return true;

    }
}
