<?php

class CustomModuleAPI extends ModuleApi
{

    private $aclCheckOptions = array('source' => 'module_api');
    public $licenseInfo;

    public function registerApiRest()
    {
        return array(
            'create' => array(
                'reqType' => 'POST',
                'path' => array('lnkdn_nav_config'),
                'pathVars' => array('module'),
                'method' => 'createRecord',
                'shortHelp' => 'This method creates a new record of the specified type',
                'longHelp' => 'include/api/help/module_post_help.html',
            ),
            'update' => array(
                'reqType' => 'PUT',
                'path' => array('lnkdn_nav_config', '?'),
                'pathVars' => array('module', 'record'),
                'method' => 'updateRecord',
                'shortHelp' => 'This method updates a record of the specified type',
                'longHelp' => 'include/api/help/module_record_put_help.html',
            ),

            'retrieve' => array(
                'reqType' => 'GET',
                'path' => array('lnkdn_nav_config','?'),
                'pathVars' => array('module','record'),
                'method' => 'retrieveRecord',
                'shortHelp' => 'Returns a single record',
                'longHelp' => 'include/api/help/module_record_get_help.html',
            ),
        );
    }

    /**
     * Creates new record of the given module and returns its formatted representation
     *
     * @param ServiceBase $api
     * @param array $args API arguments
     *
     * @return array Formatted representation of the bean
     * @throws SugarApiExceptionInvalidParameter
     * @throws SugarApiExceptionMissingParameter
     * @throws SugarApiExceptionNotAuthorized
     */
    public function createRecord(ServiceBase $api, array $args)
    {
//        $this->validateLicense($args);
        throw new SugarApiException(translate('LBL_LICENSE_EXCEEDED', $args['module']));
        $bean = $this->createBean($api, $args);
        $data = $this->formatBeanAfterSave($api, $args, $bean);

        return $data;
    }

    protected function validateLicense($args, $bean = null)
    {


        if ($bean->enabled === $args['enabled'] || $args['enabled'] == "0") {
            return true;
        } else {
            $this->getLicenseInfo();

            if (!$this->licenseInfo->hasLinkedinLicense) {
                throw new SugarApiExceptionInvalidParameter(translate('LBL_WITHOUT_LICENSE', $args['module']));
            }

            if ($this->licenseInfo->linkedinExpired) {
                throw new SugarApiExceptionInvalidParameter(translate('LBL_LICENSE_EXPIRED', $args['module']));
            }

            if ($this->licenseInfo->licenseExceeded) {
                throw new SugarApiExceptionInvalidParameter(translate('LBL_LICENSE_EXCEEDED', $args['module']));
            }

            if($bean->is_team == 1){
                $this->validateTeamEnable($bean);

            }
        }
    }

    protected function validateTeamEnable($bean){

        if (!class_exists('ManageDashboardsWithTeams')) {
            require_once("modules/lnkdn_nav_config/ManageDashboardsWithTeams.php");
        }

        $teamManagement = new ManageDashboardsWithTeams();
        $users = $teamManagement->getAvailableUsersForUpdate($bean->team_id_c);

        $usersToUpdate = count($users);
        $totalUsersEnabled = $this->licenseInfo->linkedinUsersEnabled;
        $totalSeats = $this->licenseInfo->linkedinUsers;

        if($usersToUpdate > ($totalSeats-$totalUsersEnabled)){
            throw new SugarApiExceptionInvalidParameter(translate('LBL_LICENSE_WILL_BE_EXCEEDED', $bean->module_name));
        }
    }

    protected function getLicenseInfo()
    {
        $administration = new Administration();
        $licenseInfo = $administration->retrieveSettings("license")->settings["license_linkedinInfo"];

        if (is_string($licenseInfo)) {

            // For Sugar 7.10 and bellow
            $this->licenseInfo = json_decode($licenseInfo);
        } else {

            // For Sugar 7.11 and above
            $this->licenseInfo = (object) $licenseInfo;
        }

    }

    public function updateRecord($api, $args)
    {
        foreach ($this->disabledUpdateFields as $field) {
            if (isset($args[$field])) {
                unset($args[$field]);
            }
        }

        $api->action = 'view';
        $this->requireArgs($args, array('module', 'record'));

        $bean = $this->loadBean($api, $args, 'save', $this->aclCheckOptions);

        $this->validateLicense($args, $bean);
        $api->action = 'save';

        // If we uploaded files during the record update, move them from
        // the temporary folder to the configured upload folder.
        // FIXME Moving temporary files will be handled better in BR-2059.
        $this->moveTemporaryFiles($args, $bean);
        $this->updateBean($bean, $api, $args);

        $this->updateRelatedRecords($api, $bean, $args);

        return $this->getLoadedAndFormattedBean($api, $args);
    }

    public function retrieveRecord($api, $args)
    {
        throw new SugarApiExceptionNotFound();
    }
}