<?php
/*
 * Your installation or use of this SugarCRM file is subject to the applicable
 * terms available at
 * http://support.sugarcrm.com/Resources/Master_Subscription_Agreements/.
 * If you do not agree to all of the applicable terms or do not have the
 * authority to bind the entity as an authorized representative, then do not
 * install or use this SugarCRM file.
 *
 * Copyright (C) SugarCRM Inc. All rights reserved.
 */

namespace Sugarcrm\modules\lnkdn_nav_config\clients\base\api;

use Administration;
use BeanFactory;
use Doctrine\DBAL\Query\QueryBuilder;
use SugarApiExceptionInvalidParameter;
use MetaDataManager;

class LicenseInfo
{
    private $licenseKey;
    private $licenseUrl = "https://authenticate.sugarcrm.com/rest/subscription/";
    private $productName = "Sugar Connector for LinkedIn Sales Navigator";

    private $whiteListedAttributes = array(
        "expiration_date",
        "account_id",
        "account_name",
        "status",
        "quantity_c",
        "product",
        "addons"
    );

    public function formatData(\stdClass $subscription, $numberOfEnabledUsers)
    {
        $dataToReturn = array();

        foreach ($this->whiteListedAttributes as $attribute) {
            if ($attribute === "addons") {
                foreach ($subscription->$attribute as $addon) {
                    if ($addon->product_name === $this->productName) {
                        $dataToReturn['productName'] = $this->productName;
                        $dataToReturn['hasLinkedinLicense'] = true;
                        $dataToReturn['linkedinExpirationDate'] = date("Y-m-d H:i:s", $addon->expiration_date);
                        $dataToReturn['linkedinExpired'] = (time() > $addon->expiration_date);
                        $dataToReturn['linkedinUsers'] = $addon->quantity;
                        $dataToReturn['linkedinUsersEnabled'] = $numberOfEnabledUsers['users_count'];
                        $dataToReturn['licenseExceeded'] = ($numberOfEnabledUsers['users_count'] >= $addon->quantity);
                        
                        break;
                    } else {
                        $dataToReturn['hasLinkedinLicense'] = false;
                    }
                }
            } else {
                $dataToReturn[$attribute] = $subscription->$attribute;
            }
        }

        return $dataToReturn;
    }

    public function getUsersCount()
    {
        $bean = BeanFactory::getBean("lnkdn_nav_config");

        $conn = $bean->db->getConnection();
        $builder = $conn->createQueryBuilder();
        $query = $builder
            ->select('COUNT(*) AS users_count')
            ->from($bean->table_name)
            ->where($this->getConfigWhere($builder));

        $stmt = $query->execute();

        $row = $stmt->fetch();

        return $row;
    }

    protected function getConfigWhere(QueryBuilder $builder)
    {
        $where = $builder->expr()->andX(
            $builder->expr()->eq('is_team', $builder->createPositionalParameter(0)),
            $builder->expr()->eq('enabled', $builder->createPositionalParameter(1)),
            $builder->expr()->eq('deleted', $builder->createPositionalParameter(0))
        );

        return $where;
    }

    public function getLicenseKey()
    {
        $administration = new Administration();
        $licenseInfo = $administration->retrieveSettings("license");
        $licenseKey = $licenseInfo->settings["license_key"];
        $this->licenseKey = $licenseKey;
    }

    public function callLicenseServer()
    {
        $httpClient = new \Zend_Http_Client($this->licenseUrl . $this->licenseKey);
        $request = $httpClient->request("GET");
        $response = json_decode($request->getBody());

        if ($response->success === true) {
            return $response->subscription;
        } else {
            throw new SugarApiExceptionInvalidParameter($response->error);
        }
    }

    public function getPlatform($platform)
    {
        // if the platform is not a valid registered platform, default it back to base
        $platforms = MetaDataManager::getPlatformList();

        if (!in_array($platform, $platforms)) {
            $platform = 'base';
        }

        return $platform;
    }
}
