<?php
/*
 * Your installation or use of this SugarCRM file is subject to the applicable
 * terms available at
 * http://support.sugarcrm.com/Resources/Master_Subscription_Agreements/.
 * If you do not agree to all of the applicable terms or do not have the
 * authority to bind the entity as an authorized representative, then do not
 * install or use this SugarCRM file.
 *
 * Copyright (C) SugarCRM Inc. All rights reserved.
 */

require_once 'LicenseInfo.php';

use Sugarcrm\modules\lnkdn_nav_config\clients\base\api\LicenseInfo;

class LicenseInfoApi extends SugarApi
{

    /** @var  LicenseInfo $licenseInfo */
    protected $licenseInfo;

    public function __construct()
    {
        $this->licenseInfo = new LicenseInfo();
    }

    public function registerApiRest()
    {
        return array(
            'getLicenseInfo' => array(
                'reqType' => 'GET',
                'path' => array('license-info'),
                'pathVars' => array(''),
                'method' => 'getLicenseInfo',
                'shortHelp' => 'Get license info for Linkedin',
                'longHelp' => '',
            )
        );
    }

    public function getLicenseInfo(ServiceBase $api, array $args)
    {
        try {
            $this->licenseInfo->getLicenseKey();
            $subscription = $this->licenseInfo->callLicenseServer();
            $numberOfEnabledUsers = $this->licenseInfo->getUsersCount();

            $formatedData = $this->licenseInfo->formatData($subscription, $numberOfEnabledUsers);
            $this->saveFormatedData($api, $formatedData);
            return $formatedData;
        } catch (Exception $e) {
            throw new SugarApiExceptionInvalidParameter($e->getMessage());
        }
    }

    protected function saveFormatedData(ServiceBase $api, $data)
    {
        $admin = BeanFactory::newBean('Administration');
        $platform = $this->licenseInfo->getPlatform($api->platform);
        $admin->saveSetting("license", "linkedinInfo", json_encode($data), $platform);
    }

}
