({

    initialize: function (options) {
        this._super('initialize', [options]);

        var linkedinConfigParams = App.metadata.getModule(this.module, "config") || {client_id: "", client_secret : ""};
        this.configModel = app.data.createBean(this.module, {"client_id": linkedinConfigParams.client_id || "", "client_secret": linkedinConfigParams.client_secret || ""});

        this.context.on("button:save_config:click", _.bind(this.saveConfig, this));
        this.configModel.addValidationTask('validate_api_key', _.bind(this._doValidateApiKey, this));
        this.configModel.addValidationTask('validate_client_id', _.bind(this._doValidateClientID, this));
        this.configModel.on('error:validation', this.enableButtons, this);

        this.configModel.on('change:client_id', this.enableButtons, this);
        this.configModel.on('change:client_secret', this.enableButtons, this);

        this.moduleLangObj = {
            module: this.module
        };
    },

    _doValidateClientID: function (fields, errors, callback) {
        if (_.isEmptyValue(this.configModel.get('client_id'))) {
            errors['client_id'] = errors['client_id'] || {};
            errors['client_id'].required = true;
        }

        callback(null, fields, errors);
    },

    _doValidateApiKey: function (fields, errors, callback) {
        if (_.isEmptyValue(this.configModel.get('client_secret'))) {
            errors['client_secret'] = errors['client_secret'] || {};
            errors['client_secret'].required = true;
        }

        callback(null, fields, errors);
    },

    saveConfig: function () {
        this.disableButtons();
        var url = app.api.buildURL(this.module, 'config');
        async.waterfall([
            _.bind(this.validateModel, this),
            _.bind(this.saveModel, this)
        ], _.bind(function (error) {
            this.enableButtons();
            if (error && error.status == 412 && !error.request.metadataRetry) {
                this.handleMetadataSyncError(error);
            }
        }, this));
    },

    /**
     * Check to see if all fields are valid
     * @param callback
     */
    validateModel: function (callback) {
        this.configModel.doValidate(this.getFields(this.module), function (isValid) {
            callback(!isValid);
        });
    },

    /**
     * Save config params
     * @param callback
     */
    saveModel: function () {
        var url = app.api.buildURL(this.module, 'config');

        app.api.call('create', url, this.configModel.attributes, {
                success: _.bind(function() {
                    this.showSavedConfirmation();
                    if (app.drawer.count()) {
                        // close the drawer and return to Opportunities
                        app.drawer.close(this.context, this.context.get('model'));
                        // Config changed... reload metadata
                    } else {
                        app.router.navigate(this.module, {trigger: true});
                    }

                    app.sync();
                }, this),
                error: _.bind(function() {
                    this.enableButtons();
                }, this)
            }
        );
    },

    render: function () {
        this._super("render");
    },

    /**
     * Disable buttons
     */
    disableButtons: function () {
        this.toggleButtons(false);
    },

    /**
     * Enable buttons
     */
    enableButtons: function () {
        this.toggleButtons(true);
    },

    toggleButtons: function (enabled) {
        var buttons = this.$(".btn");

        _.each(buttons, function (button) {
            if (enabled) {
                $(button).removeClass("disabled");
            }
            else {
                $(button).addClass("disabled");
            }
        })
    },

    /**
     * Show the saved confirmation alert
     *
     * @param {Object|Undefined} [onClose] the function fired upon closing.
     */
    showSavedConfirmation: function(onClose) {
        onClose = onClose || function() {};
        var alert = app.alert.show('module_config_success', {
            level: 'success',
            title: 'Linkedin Settings:',
            messages: app.lang.get('LBL_CONFIG_MODULE_SETTINGS_SAVED', " ", ""),
            autoClose: true,
            autoCloseDelay: 10000,
            onAutoClose: _.bind(function() {
                alert.getCloseSelector().off();
                onClose();
            })
        });
        var $close = alert.getCloseSelector();
        $close.on('click', onClose);
        app.accessibility.run($close, 'click');
    },
});