/*
 * This file is subject to the terms and conditions of the SugarCRM Master Subscription Agreement (MSA)
 * except that this file is provided “as is” without warranty and no support will be provided by SugarCRM for
 * these files. The MSA is located at:
 * http://support.sugarcrm.com/06_Customer_Center/10_Master_Subscription_Agreements/, unless the entity
 * receiving this file and SugarCRM have entered into a separate mutually signed agreement, which would
 * govern instead. By installing or using these files you are agreeing to the foregoing.
 *
 * @package LinkedInSalesNav
 *
 * Layout for filtering a collection.
 *
 * Composed of a module dropdown(optional), a filter dropdown and an input.
 *
 * @class View.Layouts.Base.CustomFilterLayout
 * @alias SUGAR.App.view.layouts.BaseCustomFilterLayout
 * @extends View.Layouts.Base.FilterLayout
 */
({
    extendsFrom: "FilterLayout",

    initialize: function (options) {
        this._super("initialize", [options]);
    },

    buildFilterDef: function (oSelectedFilter, searchTerm, context) {
        var filterDef = this._super('buildFilterDef', [oSelectedFilter, searchTerm, context]);


        if (this.options.module && this.options.module === "lnkdn_nav_config") {
            var component = {module: this.module, meta: {last_state: {id: "last_tab"}}}
            var lastStateKey = app.user.lastState.key('linkedin-filters', component);
            var lastView = App.user.lastState.get(lastStateKey);
            var extraFilter = this.getMyFilterDef(lastView);

            if (!_.isEmpty(extraFilter)) {
                if (_.isEmpty(filterDef)) {
                    filterDef = extraFilter;
                } else {
                    _.each(extraFilter, function (myFilter) {
                        filterDef.push(myFilter);
                    });
                }
            }
        }
        return filterDef;
    },

    getMyFilterDef: function (tab) {
        var myFilter = [];

        switch (tab) {
            case "users":
                myFilter.push({is_team: 0});
                break;
            case "teams":
            default:
                myFilter.push({is_team: 1});
                break;
        }

        return myFilter;
    }
})
