/*
 * This file is subject to the terms and conditions of the SugarCRM Master Subscription Agreement (MSA)
 * except that this file is provided “as is” without warranty and no support will be provided by SugarCRM for
 * these files. The MSA is located at:
 * http://support.sugarcrm.com/06_Customer_Center/10_Master_Subscription_Agreements/, unless the entity
 * receiving this file and SugarCRM have entered into a separate mutually signed agreement, which would
 * govern instead. By installing or using these files you are agreeing to the foregoing.
 *
 * @package LinkedInSalesNav
 */

({
    plugins: ['Dashlet'],
    clientId: "",
    clientSecret: "",
    dataLoaded: false,
    misconfigured: false,

    initialize: function (options) {
        this._super('initialize', [options]);

        this.clientSecret = app.metadata.getModule("lnkdn_nav_config", 'config').client_secret;
        this.apiKey = app.metadata.getModule("lnkdn_nav_config", 'config').client_id;
    },

    initDashlet: function () {

        if (!this.clientSecret && !this.apiKey) {
            this.misconfigured = true;

            this.render();
            return false;
        }

        this.validateLicense();
    },

    validateLicense: function () {
        var view = this;
        app.api.call('read', app.api.buildURL("license-info"), null, {
            success: function (data) {
                if(data.hasLinkedinLicense && !data.linkedinExpired){
                    view.validateEnable();
                }
                else {
                    view.noLicense = true;
                }

                view.dataLoaded = true;
                view.render();
            },
            error: function (error) {
                if (error && error.status == 412 && !error.request.metadataRetry) {
                    view.handleMetadataSyncError(error);
                }
            }
        })
    },

    validateEnable: function () {
        //check if the dashboard is enabled for current user.
        var view = this;
        var getParams = {
            "filter": [
                {enabled: 1},
                {is_team: 0},
                {user_id_c: app.user.get("id")}
            ]
        }

        app.api.call('read', app.api.buildURL("lnkdn_nav_config", null, null, getParams), null, {
            success: function (data) {
                if (data.records.length > 0) {
                    view.generateDashlet();
                }
                else {
                    view.noAccess = true;
                }
                view.dataLoaded = true;
                view.render();
            },
            error: function (data) {
            }
        })
    },

    generateDashlet: function () {
        if (!this.dashModel.get("config")) {
            var primaryEmail = _.findWhere(this.model.get("email"), {primary_address: true}) || {};
            var apiParams = this.prepareApiParams();
            this.availableSidebarHeight = ($('.dashboard').height()) - 65;
            this.linkedInUrl = encodeURI(
                apiParams.url + "&" +
                "apiKey=" + this.apiKey + "&" +
                "clientSecret=" + this.clientSecret + "&" +
                "profile=" + apiParams.profile + "&" +
                "firstName=" + this.model.get("first_name") + "&" +
                "lastName=" + this.model.get("last_name") + "&" +
                "title=" + apiParams.title + "&" +
                "email=" + primaryEmail.email_address + "&" +
                "companyWebsite=" + (this.model.get("website") || "") + "&" +
                "companyName=" + encodeURI(apiParams.companyName) + "&" +
                "name=" + encodeURI(apiParams.name) + "&" +
                "crmRecordId=" + this.model.get('id').replace(/-/g, "") + "&" +
                "crmOrgId=" + (app.config.uniqueKey || apiParams.name) + "&" +
                "crmEdition=" + apiParams.crmEdition + "&" +
                "crmSourceType=SUGAR&" +
                "width=600"
            );
        }
    },

    prepareApiParams: function () {
        var serverConfig = app.metadata.getServerInfo();

        var params = {
            profile: '',
            name: '',
            companyName: '',
            url: '',
            crmEdition: serverConfig.product_name.replace("SugarCRM", "").trim() || ""
        };

        switch (this.module) {
            case "Accounts":
                params.profile = "account";
                params.name = this.model.get("name");
                params.companyName = this.model.get("name");
                params.url = "https://www.linkedin.com/sales/widget/company-matching/?from=crm"; // company is different
                break;
            case "Opportunities":
                params.profile = "account";
                params.name = this.model.get("account_name");
                params.companyName = this.model.get("account_name");
                params.url = "https://www.linkedin.com/sales/widget/company-matching/?from=crm"; // company is different
                break;
            case "Leads":
            case "Contacts":
                params.profile = "lead";
                params.name = this.model.get("full_name");
                params.title = this.model.get("title");
                params.companyName = this.model.get("account_name");
                params.url = "https://www.linkedin.com/sales/widget/profile-matching/?from=crm"; // profile is different
                break;
            default:
                break;
        }

        return params;
    },

    render: function () {
        if(this.layout.getComponent("dashlet-toolbar")){
            this.layout.getComponent("dashlet-toolbar").hide();
        }

        this._super("render");

        this.$el.css("min-height", (this.availableSidebarHeight + 5) + "px");
        this.$el.css("width", "100%");
    },
})
