<?php
/*
 * Your installation or use of this SugarCRM file is subject to the applicable
 * terms available at
 * http://support.sugarcrm.com/Resources/Master_Subscription_Agreements/.
 * If you do not agree to all of the applicable terms or do not have the
 * authority to bind the entity as an authorized representative, then do not
 * install or use this SugarCRM file.
 *
 * Copyright (C) SugarCRM Inc. All rights reserved.
 */

require_once 'modules/lnkdn_nav_config/clients/base/api/LicenseInfoApi.php';

use Sugarcrm\modules\lnkdn_nav_config\clients\base\api\LicenseInfo;

class CheckLiNavLicenseHook
{
    /** @var  LicenseInfo $licenseInfo */
    protected $licenseInfo;

    public function __construct()
    {
        $this->licenseInfo = new LicenseInfo();
    }

    public function checkLiUserLicense($bean, $event, $arguments)
    {
        $alreadySavedTeamRecord = $this->getLiTeamRecord($bean->team_id);
        if (empty($alreadySavedTeamRecord)) {
            return;
        }

        $liTeamRecord = reset($alreadySavedTeamRecord);
        // Return if the dashboard is not enabled for the team.
        if ($liTeamRecord['enabled'] == '0') {
            return;
        }

        $alreadySavedUserRecord = $this->getLiUserRecord($bean->user_id);
        if (empty($alreadySavedUserRecord)) {
            return;
        }

        // The LI dashboard record was overridden from a user perspective.
        if ($alreadySavedUserRecord[0]['last_edited_from'] === 'users') {
            return;
        }

        // Retrieve details about available seats on the LI license.
        $this->licenseInfo->getLicenseKey();
        $subscription = $this->licenseInfo->callLicenseServer();
        $numberOfEnabledUsers = $this->licenseInfo->getUsersCount();

        $formatedData = $this->licenseInfo->formatData($subscription, $numberOfEnabledUsers);
        $this->updateLiUserAccess($alreadySavedUserRecord[0]['id'], $liTeamRecord, $formatedData['licenseExceeded']);
    }

    public function removeLiUserAccess($bean, $event, $arguments)
    {
        $userId = $arguments['related_id'];
        $alreadySavedUserRecord = $this->getLiUserRecord($userId);
        if (empty($alreadySavedUserRecord)) {
            return;
        }

        $hasAccess = $this->getLiNavAccessOnTeam($userId);
        if ($hasAccess === true) {
            return;
        }

        // We continue with the removal of access only if none of the user's teams have access to it.
        $liNavBean = BeanFactory::getBean('lnkdn_nav_config', $alreadySavedUserRecord[0]['id']);
        $liNavBean->enabled = 0;
        $liNavBean->last_edited_from = 'teams';
        $liNavBean->triggered_from_team_context = false;
        $liNavBean->save();
    }

    private function updateLiUserAccess($beanId, $liTeamDetails, $licenseExceeded)
    {
        $liNavBean = BeanFactory::getBean('lnkdn_nav_config', $beanId);
        $liNavBean->enabled = 0;
        if ($licenseExceeded === false) {
            $liNavBean->enabled = 1;
        }
        $liNavBean->accounts = $liTeamDetails['accounts'];
        $liNavBean->contacts = $liTeamDetails['contacts'];
        $liNavBean->leads = $liTeamDetails['leads'];
        $liNavBean->opportunities = $liTeamDetails['opportunities'];

        $liNavBean->last_edited_from = 'teams';
        $liNavBean->triggered_from_team_context = true;

        $liNavBean->save();
    }

    private function getLiUserRecord($userId)
    {
        $lnkdn = new SugarQuery();
        $lnkdn->from(BeanFactory::newBean('lnkdn_nav_config'));
        $lnkdn->select(array('id', 'last_edited_from'));
        $lnkdn->where()->equals('user_id_c', $userId);

        return $lnkdn->execute();
    }

    private function getLiTeamRecord($teamId)
    {
        $lnkdn = new SugarQuery();
        $lnkdn->from(BeanFactory::newBean('lnkdn_nav_config'));
        $lnkdn->where()->equals('team_id_c', $teamId);

        return $lnkdn->execute();
    }

    /**
     * We follow a most permissive strategy concerning access to LinkedIn dashboard based on teams.
     *
     * If the user is in multiple teams and at least one of them has the LinkedIn dashboard enabled,
     * then the user should keep his access to the dashboard.
     *
     * @param string $userId
     * @return bool
     */
    private function getLiNavAccessOnTeam($userId)
    {
        $hasAccess = false;
        /** @var User $userBean */
        $userBean = BeanFactory::getBean('Users', $userId);
        $userTeams = $userBean->get_my_teams();

        // Filter user teams.
        if ($userTeams[1] === 'Global') {
            // Remove the Global team
            unset($userTeams[1]);
        }
        // Remove the private team
        unset($userTeams[$userBean->getPrivateTeamID()]);

        if (count($userTeams) === 0) {
            return $hasAccess;
        }

        // Iterate over the remaining teams and check if at least one of them has the LinkedIn Sales dashboard enabled.
        foreach ($userTeams as $team) {
            $liTeamRecord = $this->getLiTeamRecord($team);
            if ($liTeamRecord[0]['enabled'] === '1') {
                $hasAccess = true;
                break;
            }
        }

        return $hasAccess;
    }
}
