<?php
if (!defined('sugarEntry') || !sugarEntry) {
    die('Not A Valid Entry Point');
}

require_once('modules/Administration/QuickRepairAndRebuild.php');
require_once('include/SugarQuery/SugarQuery.php');
require_once('data/BeanFactory.php');

/**
 * post_install method
 */
function post_install()
{
    $admin = BeanFactory::getBean('Administration');
    // Save this dummy value in order to enable the /config route for <lnkdn_nav_config> module.
    $admin->saveSetting('lnkdn_nav_config', 'dummy_key_to_enable_config_route', 'dummy_value', 'base');

    $autoexecute = false; //execute the SQL
    $show_output = true; //output to the screen
    $randc = new RepairAndClear();
    $randc->repairAndClearAll(array('rebuildExtensions', 'clearAdditionalCaches'), array(translate('LBL_ALL_MODULES')),
        $autoexecute, $show_output);

    createLinkedInNavUserRecords();
    createLinkedInNavTeamRecords();
}

/**
 * Creates a <lnkdn_nav_config> record for each existing user in the system.
 */
function createLinkedInNavUserRecords() {
    $lnkdn = new SugarQuery();
    $lnkdn->from(BeanFactory::newBean('lnkdn_nav_config'));
    $lnkdn->select(array('user_id_c'));
    $lnkdn->distinct(true);
    $alreadyConfiguredUsers = $lnkdn->execute();

    $users = new SugarQuery();
    $users->from(BeanFactory::newBean('Users'));
    $users->select(array('id', 'full_name', 'first_name', 'last_name'));
    $users->distinct(true);
    if (!empty($alreadyConfiguredUsers)) {
        $existingIds = array();
        foreach ($alreadyConfiguredUsers as $user) {
            $existingIds[] = $user['user_id_c'];
        }
        $users->where()->notIn('id', $existingIds);
    }
    $userResults = $users->execute();

    foreach ($userResults as $userDetails) {
        $newBean = BeanFactory::newBean('lnkdn_nav_config');
        $newBean->name = $userDetails['full_name'];
        if (!empty($userDetails['first_name']) && !empty($userDetails['last_name'])) {
            $newBean->name = $userDetails['first_name'] . ' ' . $userDetails['last_name'];
        }
        $newBean->user_id_c = $userDetails['id'];
        $newBean->last_edited_from = 'teams'; // We don't consider the record managed from a TEAM's perspective.
        $newBean->is_team = false;
        $newBean->triggered_from_team_context = true;
        $newBean->save();
        unset($newBean);
    }
}

function createLinkedInNavTeamRecords() {
    $lnkdn = new SugarQuery();
    $lnkdn->from(BeanFactory::newBean('lnkdn_nav_config'));
    $lnkdn->select(array('team_id_c'));
    $lnkdn->distinct(true);
    $lnkdn->where()->isNotEmpty('team_id_c');
    $alreadyConfiguredTeams = $lnkdn->execute();

    $teams = new SugarQuery();
    $teams->from(BeanFactory::newBean('Teams'));
    $teams->select(array('id', 'name'));
    $teams->distinct(true);
    $teams->where()->equals('private', '0');
    $teams->where()->notEquals('name', 'Global');
    if (!empty($alreadyConfiguredTeams)) {
        $existingIds = array();
        foreach ($alreadyConfiguredTeams as $team) {
            $existingIds[] = $team['team_id_c'];
        }
        $teams->where()->notIn('id', $existingIds);
    }
    $teamResults = $teams->execute();

    foreach ($teamResults as $teamDetails) {
        $newBean = BeanFactory::newBean('lnkdn_nav_config');
        $newBean->name = $teamDetails['name'];
        $newBean->team_id_c = $teamDetails['id'];
        $newBean->last_edited_from = 'teams';
        $newBean->is_team = true;
        $newBean->save();
        unset($newBean);
    }
}
